@@ -24,12 +24,15 @@ import time
 import mock
 from os_win import exceptions
 from os_win import utilsfactory
+from oslo_config import cfg
 
 from hyperv.neutron import constants
 from hyperv.neutron import exception
 from hyperv.neutron import hyperv_neutron_agent
 from hyperv.tests import base
 
+CONF = cfg.CONF
+
 
 class TestHyperVNeutronAgent(base.BaseTestCase):
 
@@ -55,34 +58,66 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
         self.agent._nvgre_ops = mock.MagicMock()
         self.agent._workers = mock.MagicMock()
 
-    @mock.patch.object(hyperv_neutron_agent, 'synchronized')
-    def test_port_synchronized(self, mock_synchronized):
-        fake_method_side_effect = mock.Mock()
-
-        @hyperv_neutron_agent._port_synchronized
-        def fake_method(fake_arg, port_id):
-            fake_method_side_effect(fake_arg, port_id)
-
-        mock_synchronized.return_value = lambda x: x
-        expected_lock_name = 'port-lock-%s' % mock.sentinel.port_id
-
-        fake_method(fake_arg=mock.sentinel.arg, port_id=mock.sentinel.port_id)
-        mock_synchronized.assert_called_once_with(expected_lock_name)
-        fake_method_side_effect.assert_called_once_with(
-            mock.sentinel.arg, mock.sentinel.port_id)
-
     def test_load_physical_network_mappings(self):
         test_mappings = ['fakenetwork1:fake_vswitch',
-                         'fakenetwork2:fake_vswitch_2', '*:fake_vswitch_3']
-        expected = [('fakenetwork1', 'fake_vswitch'),
-                    ('fakenetwork2', 'fake_vswitch_2'),
-                    ('.*', 'fake_vswitch_3')]
+                         'fakenetwork2:fake_vswitch_2', '*:fake_vswitch_3',
+                         'bad_mapping']
+        expected = [('fakenetwork1$', 'fake_vswitch'),
+                    ('fakenetwork2$', 'fake_vswitch_2'),
+                    ('.*$', 'fake_vswitch_3')]
 
         self.agent._load_physical_network_mappings(test_mappings)
 
         self.assertEqual(expected,
                          list(self.agent._physical_network_mappings.items()))
 
+    def test_get_vswitch_for_physical_network_with_default_switch(self):
+        test_mappings = ['fakenetwork:fake_vswitch',
+                         'fakenetwork2$:fake_vswitch_2',
+                         'fakenetwork*:fake_vswitch_3']
+        self.agent._load_physical_network_mappings(test_mappings)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork')
+        self.assertEqual('fake_vswitch', physnet)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork2$')
+        self.assertEqual('fake_vswitch_2', physnet)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork3')
+        self.assertEqual('fake_vswitch_3', physnet)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork35')
+        self.assertEqual('fake_vswitch_3', physnet)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fake_network1')
+        self.assertEqual('fake_network1', physnet)
+
+    def test_get_vswitch_for_physical_network_without_default_switch(self):
+        test_mappings = ['fakenetwork:fake_vswitch',
+                         'fakenetwork2:fake_vswitch_2']
+        self.agent._load_physical_network_mappings(test_mappings)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork')
+        self.assertEqual('fake_vswitch', physnet)
+
+        physnet = self.agent._get_vswitch_for_physical_network('fakenetwork2')
+        self.assertEqual('fake_vswitch_2', physnet)
+
+    def test_get_vswitch_for_physical_network_none(self):
+        test_mappings = ['fakenetwork:fake_vswitch',
+                         'fakenetwork2:fake_vswitch_2']
+        self.agent._load_physical_network_mappings(test_mappings)
+
+        physnet = self.agent._get_vswitch_for_physical_network(None)
+        self.assertEqual('', physnet)
+
+        test_mappings = ['fakenetwork:fake_vswitch',
+                         'fakenetwork2:fake_vswitch_2', '*:fake_vswitch_3']
+        self.agent._load_physical_network_mappings(test_mappings)
+
+        physnet = self.agent._get_vswitch_for_physical_network(None)
+        self.assertEqual('fake_vswitch_3', physnet)
+
     @mock.patch.object(hyperv_neutron_agent.nvgre_ops, 'HyperVNvgreOps')
     def test_init_nvgre_disabled(self, mock_hyperv_nvgre_ops):
         self.agent._init_nvgre()
@@ -130,6 +165,22 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
         self.assertIsNone(network)
         self.assertIsNone(port_map)
 
+    @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
+                       '_reclaim_local_network')
+    def test_network_delete(self, mock_reclaim_local_network):
+        self.agent._network_vswitch_map[mock.sentinel.net_id] = (
+            mock.sentinel.vswitch)
+
+        self.agent.network_delete(mock.sentinel.context, mock.sentinel.net_id)
+        mock_reclaim_local_network.assert_called_once_with(
+            mock.sentinel.net_id)
+
+    @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
+                       '_reclaim_local_network')
+    def test_network_delete_not_defined(self, mock_reclaim_local_network):
+        self.agent.network_delete(mock.sentinel.context, mock.sentinel.net_id)
+        self.assertFalse(mock_reclaim_local_network.called)
+
     @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
                        '_treat_vif_port')
     def test_port_update_not_found(self, mock_treat_vif_port):
@@ -157,6 +208,19 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
             mock.sentinel.network_type, mock.sentinel.physical_network,
             mock.sentinel.segmentation_id, mock.sentinel.admin_state_up)
 
+    def test_tunnel_update(self):
+        self.agent.tunnel_update(mock.sentinel.context,
+                                 tunnel_ip=mock.sentinel.tunnel_ip,
+                                 tunnel_type=mock.sentinel.tunnel_type)
+        self.agent._nvgre_ops.tunnel_update.assert_called_once_with(
+            mock.sentinel.context, mock.sentinel.tunnel_ip,
+            mock.sentinel.tunnel_type)
+
+    def test_tunnel_update_provider_ip(self):
+        self.agent.tunnel_update(mock.sentinel.context,
+                                 tunnel_ip=CONF.NVGRE.provider_tunnel_ip)
+        self.assertFalse(self.agent._nvgre_ops.tunnel_update.called)
+
     def test_lookup_update(self):
         kwargs = {'lookup_ip': mock.sentinel.lookup_ip,
                   'lookup_details': mock.sentinel.lookup_details}
@@ -251,6 +315,13 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
             constants.TYPE_LOCAL,
             mock.sentinel.FAKE_PHYSICAL_NETWORK)
 
+    def test_reclaim_local_network(self):
+        self.agent._network_vswitch_map[mock.sentinel.net_id] = (
+            mock.sentinel.vswitch)
+
+        self.agent._reclaim_local_network(mock.sentinel.net_id)
+        self.assertNotIn(mock.sentinel.net_id, self.agent._network_vswitch_map)
+
     def test_port_bound_enable_metrics(self):
         self.agent.enable_metrics_collection = True
         self._test_port_bound(True)
@@ -396,7 +467,7 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
         self._check_treat_vif_port_state_up()
 
         sg_agent = self.agent.sec_groups_agent
-        sg_agent.prepare_devices_filter.assert_called_once_with(
+        sg_agent.refresh_firewall.assert_called_once_with(
             [mock.sentinel.port_id])
 
     def test_treat_vif_port_sg_disabled(self):
@@ -537,3 +608,27 @@ class TestHyperVNeutronAgent(base.BaseTestCase):
         # allow the threads to finish. one second is enough for a noop call.
         time.sleep(1)
         mock_fn.assert_has_calls([mock.call(mock.sentinel.parameter)] * 8)
+
+    @mock.patch('time.sleep')
+    @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
+                       '_port_enable_control_metrics')
+    @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
+                       '_treat_devices_added')
+    @mock.patch.object(hyperv_neutron_agent.HyperVNeutronAgentMixin,
+                       '_create_event_listeners')
+    def test_daemon_loop(self, mock_create_listeners, mock_treat_dev_added,
+                         mock_port_enable_metrics, mock_sleep):
+        self.agent._nvgre_enabled = True
+        mock_port_enable_metrics.side_effect = KeyError
+        mock_sleep.side_effect = KeyboardInterrupt
+
+        self.assertRaises(KeyboardInterrupt, self.agent.daemon_loop)
+
+        self.assertEqual(self.agent._utils.get_vnic_ids.return_value,
+                         self.agent._added_ports)
+        self.assertEqual(set(), self.agent._removed_ports)
+        mock_create_listeners.assert_called_once_with()
+        mock_treat_dev_added.assert_called_once_with()
+        self.agent._nvgre_ops.refresh_nvgre_records.assert_called_once_with()
+        mock_port_enable_metrics.assert_called_with()
+        self.agent._utils.update_cache.assert_called_once_with()
