@@ -20,13 +20,10 @@ Unit tests for the Hyper-V Security Groups Driver.
 import mock
 from os_win import exceptions
 from os_win import utilsfactory
-from oslo_config import cfg
 
 from hyperv.neutron import security_groups_driver as sg_driver
 from hyperv.tests import base
 
-CONF = cfg.CONF
-
 
 class SecurityGroupRuleTestHelper(base.BaseTestCase):
     _FAKE_DIRECTION = 'egress'
@@ -43,8 +40,8 @@ class SecurityGroupRuleTestHelper(base.BaseTestCase):
     _FAKE_PORT_MIN = 9001
     _FAKE_PORT_MAX = 9011
 
-    def _create_security_rule(self):
-        return {
+    def _create_security_rule(self, **rule_updates):
+        rule = {
             'direction': self._FAKE_DIRECTION,
             'ethertype': self._FAKE_ETHERTYPE,
             'protocol': self._FAKE_PROTOCOL,
@@ -54,6 +51,8 @@ class SecurityGroupRuleTestHelper(base.BaseTestCase):
             'port_range_max': self._FAKE_PORT_MAX,
             'security_group_id': self._FAKE_SG_ID
         }
+        rule.update(rule_updates)
+        return rule
 
     @classmethod
     def _acl(self, key1, key2):
@@ -89,7 +88,7 @@ class TestHyperVSecurityGroupsDriver(SecurityGroupRuleTestHelper):
         # Test without remote_group_id
         rule_list = self._driver._select_sg_rules_for_port(mock_port,
                                                            'ingress')
-        self.assertEqual(self._FAKE_SG_ID, rule_list[0]['security_group_id'])
+        self.assertNotIn('security_group_id', rule_list[0])
 
         # Test with remote_group_id
         fake_sg_template['remote_group_id'] = self._FAKE_SG_ID
@@ -97,8 +96,9 @@ class TestHyperVSecurityGroupsDriver(SecurityGroupRuleTestHelper):
                                                       [self._FAKE_MEMBER_IP]}
         rule_list = self._driver._select_sg_rules_for_port(mock_port,
                                                            'ingress')
-        self.assertEqual(self._FAKE_SG_ID, rule_list[0]['security_group_id'])
         self.assertEqual('10.0.0.1/32', rule_list[0]['source_ip_prefix'])
+        self.assertNotIn('security_group_id', rule_list[0])
+        self.assertNotIn('remote_group_id', rule_list[0])
 
         # Test for fixed 'ip' existing in 'sg_members'
         self._driver._sg_members[self._FAKE_SG_ID][self._FAKE_ETHERTYPE] = [
@@ -218,6 +218,7 @@ class TestHyperVSecurityGroupsDriver(SecurityGroupRuleTestHelper):
         self._driver.update_port_filter(new_mock_port)
 
         self.assertNotIn(new_mock_port['device'], self._driver._security_ports)
+        mock_method.assert_called_once_with(new_mock_port)
 
     def test_remove_port_filter(self):
         mock_port = self._get_port()
@@ -465,15 +466,22 @@ class SecurityGroupRuleGeneratorR2TestCase(SecurityGroupRuleR2BaseTestCase):
 
         self.assertEqual(expected, actual)
 
+    def test_get_rule_protocol_icmp_ipv6(self):
+        self._check_get_rule_protocol(
+            expected=self._acl('protocol', 'ipv6-icmp'),
+            protocol='icmp',
+            ethertype='IPv6')
+
     def test_get_rule_protocol_icmp(self):
-        self._check_get_rule_protocol('icmp', self._acl('protocol', 'icmp'))
+        self._check_get_rule_protocol(expected=self._acl('protocol', 'icmp'),
+                                      protocol='icmp')
 
     def test_get_rule_protocol_no_icmp(self):
-        self._check_get_rule_protocol('tcp', 'tcp')
+        self._check_get_rule_protocol(expected='tcp',
+                                      protocol='tcp')
 
-    def _check_get_rule_protocol(self, protocol, expected):
-        rule = self._create_security_rule()
-        rule['protocol'] = protocol
+    def _check_get_rule_protocol(self, expected, **rule_updates):
+        rule = self._create_security_rule(**rule_updates)
         actual = self.sg_gen._get_rule_protocol(rule)
 
         self.assertEqual(expected, actual)
